@@ -72,6 +72,7 @@ typedef struct _swClient
     uint32_t destroyed :1;
     uint32_t redirect :1;
     uint32_t http2 :1;
+    uint32_t sleep :1;
 
     /**
      * one package: length check
@@ -137,6 +138,8 @@ typedef struct _swClient
 } swClient;
 
 int swClient_create(swClient *cli, int type, int async);
+int swClient_sleep(swClient *cli);
+int swClient_wakeup(swClient *cli);
 #ifdef SW_USE_OPENSSL
 int swClient_enable_ssl_encrypt(swClient *cli);
 int swClient_ssl_handshake(swClient *cli);
