@@ -49,6 +49,7 @@ enum swErrorCode
     SW_ERROR_SSL_EMPTY_PEER_CERTIFICATE,
     SW_ERROR_SSL_VEFIRY_FAILED,
     SW_ERROR_SSL_BAD_CLIENT,
+    SW_ERROR_SSL_BAD_PROTOCOL,
 
     SW_ERROR_PACKAGE_LENGTH_TOO_LARGE = 1201,
     SW_ERROR_DATA_LENGTH_TOO_LARGE,
@@ -73,6 +74,8 @@ enum swErrorCode
     SW_ERROR_HTTP_PROXY_HANDSHAKE_ERROR = 8001,
     SW_ERROR_HTTP_INVALID_PROTOCOL,
 
+    SW_ERROR_WEBSOCKET_BAD_CLIENT = 8501,
+
     /**
      * server global error
      */
