@@ -171,6 +171,42 @@ int swClient_create(swClient *cli, int type, int async)
     return SW_OK;
 }
 
+int swClient_sleep(swClient *cli)
+{
+    int ret;
+    if (cli->socket->events & SW_EVENT_WRITE)
+    {
+        ret = SwooleG.main_reactor->set(SwooleG.main_reactor, cli->socket->fd, cli->socket->fdtype | SW_EVENT_WRITE);
+    }
+    else
+    {
+        ret = SwooleG.main_reactor->del(SwooleG.main_reactor, cli->socket->fd);
+    }
+    if (ret)
+    {
+        cli->sleep = 1;
+    }
+    return ret;
+}
+
+int swClient_wakeup(swClient *cli)
+{
+    int ret;
+    if (cli->socket->events & SW_EVENT_WRITE)
+    {
+        ret = SwooleG.main_reactor->set(SwooleG.main_reactor, cli->socket->fd, cli->socket->fdtype | SW_EVENT_READ | SW_EVENT_WRITE);
+    }
+    else
+    {
+        ret = SwooleG.main_reactor->add(SwooleG.main_reactor, cli->socket->fd, cli->socket->fdtype | SW_EVENT_READ);
+    }
+    if (ret)
+    {
+        cli->sleep = 0;
+    }
+    return ret;
+}
+
 #ifdef SW_USE_OPENSSL
 int swClient_enable_ssl_encrypt(swClient *cli)
 {
@@ -698,12 +734,9 @@ static int swClient_tcp_recv_no_buffer(swClient *cli, char *data, int len, int f
             {
                 continue;
             }
-            else
-            {
-                break;
-            }
         }
 #endif
+        break;
     }
 
     return ret;
