@@ -19,7 +19,10 @@
 
 #ifdef SW_USE_OPENSSL
 
+#include <openssl/crypto.h>
+
 static int openssl_init = 0;
+static pthread_mutex_t *lock_array;
 
 static const SSL_METHOD *swSSL_get_method(int method);
 static int swSSL_verify_callback(int ok, X509_STORE_CTX *x509_store);
@@ -40,6 +43,8 @@ static int swSSL_npn_advertised(SSL *ssl, const uchar **out, uint32_t *outlen, v
 static int swSSL_alpn_advertised(SSL *ssl, const uchar **out, uchar *outlen, const uchar *in, uint32_t inlen, void *arg);
 #endif
 
+static void swSSL_lock_callback(int mode, int type, char *file, int line);
+
 static const SSL_METHOD *swSSL_get_method(int method)
 {
     switch (method)
@@ -98,6 +103,10 @@ static const SSL_METHOD *swSSL_get_method(int method)
 
 void swSSL_init(void)
 {
+    if (openssl_init)
+    {
+        return;
+    }
 #if OPENSSL_VERSION_NUMBER >= 0x10100003L && !defined(LIBRESSL_VERSION_NUMBER)
     OPENSSL_init_ssl(OPENSSL_INIT_LOAD_CONFIG, NULL);
 #else
@@ -109,6 +118,70 @@ void swSSL_init(void)
     openssl_init = 1;
 }
 
+void swSSL_destroy()
+{
+    if (!openssl_init)
+    {
+        return;
+    }
+
+    CRYPTO_set_locking_callback(NULL);
+    int i;
+    for (i = 0; i < CRYPTO_num_locks(); i++)
+    {
+        pthread_mutex_destroy(&(lock_array[i]));
+    }
+    openssl_init = 0;
+#if OPENSSL_VERSION_NUMBER >= OPENSSL_VERSION_1_0_0
+    CRYPTO_THREADID_set_callback(NULL);
+#else
+    CRYPTO_set_id_callback(NULL);
+#endif
+    CRYPTO_set_locking_callback(NULL);
+}
+
+static void swSSL_lock_callback(int mode, int type, char *file, int line)
+{
+    if (mode & CRYPTO_LOCK)
+    {
+        pthread_mutex_lock(&(lock_array[type]));
+    }
+    else
+    {
+        pthread_mutex_unlock(&(lock_array[type]));
+    }
+}
+
+#if OPENSSL_VERSION_NUMBER >= OPENSSL_VERSION_1_0_0
+static void swSSL_id_callback(CRYPTO_THREADID * id)
+{
+    CRYPTO_THREADID_set_numeric(id, (ulong_t) pthread_self());
+}
+#else
+static ulong_t swSSL_id_callback(void)
+{
+    return (ulong_t) pthread_self();
+}
+#endif
+
+void swSSL_init_thread_safety()
+{
+    int i;
+    lock_array = (pthread_mutex_t *) OPENSSL_malloc(CRYPTO_num_locks() * sizeof(pthread_mutex_t));
+    for (i = 0; i < CRYPTO_num_locks(); i++)
+    {
+        pthread_mutex_init(&(lock_array[i]), NULL);
+    }
+
+#if OPENSSL_VERSION_NUMBER >= OPENSSL_VERSION_1_0_0
+    CRYPTO_THREADID_set_callback(swSSL_id_callback);
+#else
+    CRYPTO_set_id_callback(swSSL_id_callback);
+#endif
+
+    CRYPTO_set_locking_callback((void (*)()) swSSL_lock_callback);
+}
+
 void swSSL_server_http_advise(SSL_CTX* ssl_context, swSSL_config *cfg)
 {
 #ifdef TLSEXT_TYPE_application_layer_protocol_negotiation
